; part of the io library for the PROGS SMS C library
;   IOOpen              back to the basics open routine
;   IOOpenFor           same as IOOpen, but for another job
;   IOClose             close a channel
;   IODelete            delete a channel
;   IOFormat            format a device
;   IOTimeOutInit       allocate area to store default timeouts
;   IOTimeOutGet        routine to get default timeout for a channel
;   IOTimeOutSet        routine to set default timeout for a channel
; written by Joachim Van der Auwera for PROGS
; (c) 1994 by PROGS, Professional & Graphical Software, all rights reserved
; started 19 October 1994
        section text
        include win1_inc_qdos_io
        include win1_inc_qdos_ioa
        include win1_inc_qdos_sms
        include win1_inc_keys_sys
        include win1_inc_keys_err
        include win1_inc_aid_mac
;
; routine to open a channel
name    equ     4               ; .l ptr to filename
key     equ     8               ; .w open key
owner   equ     10              ; .l jobid (IOOpenFor only)
chido   equ     10              ; .l Channel * (for IOOpen)
chidf   equ     14              ; .l Channel * (for IOOpenFor)
_IOOpen xsub
        move.l  #myself,d1      ; this one is for myself
        move.l  chido(sp),a0    ; Channel *
        bra.s   open
_IOOpenFor xsub
        move.l  owner(sp),d1    ; job which should own this one
        move.l  chidf(sp),a0    ; Channel *
open    move.w  key(sp),d0      ; open key
        move.l  name(sp),a1     ; name of channel
        regsave d2/d3/a2        ; keep corrupted registers
        move.l  d0,d3           ; open key
        move.l  a0,a2           ; Channel *

        getssp  66              ; need some stack space for filename
        lea     2(sp),a0        ; place to copy string to
        moveq   #63,d2          ; max length of string
        moveq   #-1,d0          ; counter of string length
copystr addq.w  #1,d0           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d2,copystr
        move.w  d0,(sp)         ; set length of string

        move.l  sp,a0           ; base of string with channel name
        moveq   #ioa.open,d0    ; open channel
        trap    #do.ioa         ; do it
        tst.l   d0              ; failed ??
        bne.s   oops            ; yes -> done
        move.l  a0,(a2)         ; fill in Channel id
                                ; make sure TimeOut initialised, no PushBack
        move.l  a0,d1           ; Channel
        move.l  TimeOuts(pc),a0 ; base of table
        add.w   d1,d1           ; calculate ...
        add.w   d1,d1           ; ... offset
        moveq   #-1,d2          ; value to set
        move.w  d2,0(a0,d1.w)   ; FOREVER, no PushBack
oops    subend
;
; close a channel
chid    equ     4               ; .l channel id
_IOClose xsub
        move.l  chid(sp),a0     ; channel id
        moveq   #ioa.clos,d0    ; close a channel
        trap    #do.ioa         ; do it
        subend
;
; delete a file
_IODelete xsub
        move.l  #myself,d1      ; job which shuold own this one
        move.l  name(sp),a1     ; name of channel
        regsave d2/d3           ; keep corrupted registers

        getssp  66              ; need some stack space for filename
        lea     2(sp),a0        ; place to copy string to
        moveq   #63,d2          ; max length of string
        moveq   #-1,d0          ; counter of string length
copstr2 addq.w  #1,d0           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d2,copstr2
        move.w  d0,(sp)         ; set length of string

        move.l  sp,a0           ; base of string with file name
        moveq   #ioa.delf,d0    ; delete file
        trap    #do.ioa         ; do it
        subend
;
; format a device
good    equ     8               ; .l ptr to int, good sectors
total   equ     12              ; .l ptr to int, total sectors
_IOFormat xsub
        lea     good(sp),a0
        move.l  name(sp),a1     ; name of channel
        regsave d2/d3/a2        ; keep corrupted registers
        move.l  a0,a2           ; keep place for return

        getssp  34              ; need some stack space for filename
        lea     2(sp),a0        ; place to copy string to
        moveq   #31,d2          ; max length of string
        moveq   #-1,d0          ; counter of string length
copstr3 addq.w  #1,d0           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d2,copstr3
        move.w  d0,(sp)         ; set length of string

        move.l  sp,a0           ; base of string with file name
        moveq   #ioa.frmt,d0    ; delete file
        trap    #do.ioa         ; do it
        move.l  (a2)+,a1        ; &good
        ext.l   d1              ; make it an int
        move.l  d1,(a1)         ; fill in
        move.l  (a2)+,a1        ; &total
        ext.l   d2              ; make it an int
        move.l  d2,(a1)         ; fill in
        subend
;
; pointer to a table which contains the timeout value for all the channels
        xdef    TimeOuts
TimeOuts dc.l   0
;
; routine to initialise table with timeouts and putback characters !!
_IOTimeOutInit xsub
        regsave d2/d3/a2
        move.l  TimeOuts(pc),d0 ; is there a timeout table ??
        bne.s   tdone
        moveq   #sms.info,d0    ; get system info
        trap    #do.sms2        ; do it
                                ; calculate length of area needed for timeouts
        move.l  sys_chtt(a0),d3 ; end of channel table
        sub.l   sys_chtb(a0),d3 ; - base = length of table
        moveq   #0,d1
        move.l  d1,-(sp)        ; allocate for system
        move.l  d3,-(sp)        ; length needed
        xjsr    _MEMAllocateFor ; allocate memory
        addq.l  #8,sp           ; clean up stack
        tst.l   d0              ; allocated ??
        bne.s   fillin          ; yes -> continue
        moveq   #err.imem,d0    ; ERRMemory
        bra.s   tdone

fillin  lea     TimeOuts(pc),a0 ; place for pointer to table
        move.l  d0,(a0)         ; fill in

        move.l  d0,a0           ; base of timouts
        lsr.l   #2,d3           ; number of channels
        subq.w  #1,d3           ; adjust for dbf
        moveq   #forever,d1     ; value for init, TimeOut FOREVER
        clr.w   d1              ; no PutBack just yet
clcht   move.l  d1,(a0)+        ; clear entry
        dbf     d3,clcht
        moveq   #0,d0           ; no error
tdone   subend
;
; routine to set the timeout for a channel
channel equ     4               ; .l channel id
timeout equ     8               ; .w timeout
_IOTimeOutSet xsub
        move.l  TimeOuts(pc),a0 ; base of table
        move.l  channel(sp),d1  ; Channel
        add.w   d1,d1           ; calculate ...
        add.w   d1,d1           ; ... offset
        move.w  timeout(sp),0(a0,d1.w)  ; TimeOut
        moveq   #0,d0           ; no error
        subend
;
; routine to get the current value of the timeout for a channel
_IOTimeOutGet xsub
        move.l  TimeOuts(pc),a0 ; base of table
        move.l  channel(sp),d1  ; Channel
        add.w   d1,d1           ; calculate ...
        add.w   d1,d1           ; ... offset
        move.l  timeout(sp),a1  ; place to store timeout
        move.w  0(a0,d1.w),(a1) ; TimeOut
        moveq   #0,d0           ; no error
        subend
;
        end
