; part of the io library for the PROGS SMS C library
;   IOGet               get a byte from the channel
;   IOPut               send a byte to the channel
;   IOGetBytes          get multiple bytes from the channel
;   IOPutBytes          send multiple bytes to the channel
;   IOGetString         get a line (\0 or \n terminated) from the channel
;   IOPutString         send a line to the channel (\n added)
;   IOTest              check for pending operation (input) on file
;   IODone              check if all operations done
;   IOFlush             make sure all write operations on a file are done
;   IOTruncate          truncate the file at the current position
;   IOPositionGet       Get current file position
;   IOPositionSet       Set file position
;   IOPositionMove      move file position
; written by Joachim Van der Auwera for PROGS
; (c) 1994 by PROGS, Professional & Graphical Software, all rights reserved
; started 21 October 1994
        section text
        include win1_inc_qdos_io
        include win1_inc_keys_err
        include win1_inc_keys_k
        include win1_inc_aid_mac
;
        xref    TimeOuts
timeout macro
        move.l  a0,-(sp)
        move.l  a0,d3           ; Channel
        move.l  TimeOuts(pc),a0 ; base of table
        add.w   d3,d3           ; calculate ...
        add.w   d3,d3           ; ... offset
        clr.w   2(a0,d3.w)      ; no more PutBack character
        move.w  0(a0,d3.w),d3   ; TimeOut
        move.l  (sp)+,a0
        endm
;
; get char from channel
chid    equ     4               ; .l Channel descritor
retc    equ     8               ; .l char *
_IOGet xsub
        move.l  chid(sp),a0     ; Channel
        regsave d3/a1/a2/a3
        move.l  retc+16(sp),a2  ; place for return
                                ; get timeout
        move.l  a0,d3           ; Channel
        move.l  TimeOuts(pc),a3 ; base of table
        add.w   d3,d3           ; calculate ...
        add.w   d3,d3           ; ... offset
        tst.b   2(a3,d3.w)      ; PushBack ??
        beq.s   fbyt            ; no -> fetch byte
        move.b  3(a3,d3.w),(a2) ; set result
        clr.w   2(a3,d3.w)      ; no more PushBack
        moveq   #0,d0           ; no error
        bra.s   getdone
fbyt    move.w  0(a3,d3.w),d3   ; set TimeOut
        moveq   #iob.fbyt,d0    ; fetch byte
        trap    #do.io          ; do it
        move.b  d1,(a2)         ; set result
getdone subend
;
; send char to channel
char    equ     8               ; .w char
_IOPut xsub
        move.l  chid(sp),a0     ; Channel
        move.w  char(sp),d1     ; character
        regsave d3
        moveq   #iob.sbyt,d0    ; send a byte
        timeout
        trap    #do.io          ; do it
        subend
;
; put a byte back into the input stream !!
_IOPutBack xsub
        move.l  chid(sp),d1     ; Channel
        add.w   d1,d1           ; calculate ...
        add.w   d1,d1           ; ... offset
        move.l  TimeOuts(pc),a0 ; base of table
        move.b  char+1(sp),3(a0,d1.w) ; byte to put back
        clr.b   2(a0,d1.w)      ; PutBack on
        moveq   #0,d0           ; no error
        subend
;
; get multiple bytes from channel
length  equ     8               ; number of bytes to get
place   equ     12              ; place for return
count   equ     16              ; count of bytes received
_IOGetBytes xsub
        moveq   #iob.fmul,d0    ; fetch multiple bytes
        regsave d3/d4/d5/d7/a2/a3
        move.l  d0,d7           ; remember command to use
        moveq   #0,d0           ; no error just yet
        move.l  length+24(sp),d4; number of bytes to get/send
        beq.s   gbdone          ; nothing -> done
        move.l  count+24(sp),a2 ; place for return
        moveq   #0,d5           ; number of bytes I've got/sent
        move.l  a0,d3           ; Channel
        move.l  TimeOuts(pc),a3 ; base of table
        add.w   d3,d3           ; calculate ...
        add.w   d3,d3           ; ... offset
        tst.b   2(a3,d3.w)      ; PushBack ??
        beq.s   fbyts           ; no -> fetch byte
        move.b  3(a3,d3.w),(a2)+; set result
        clr.w   2(a3,d3.w)      ; no more PushBack
        moveq   #1,d5           ; already got one byte
        subq.l  #1,d4           ; one byte less to go
        bra.s   gbloop          ; continue
fbyts   move.w  0(a3,d3.w),d3   ; TimeOut
        bra.s   gbloop
;
; send multiple bytes to channel
_IOPutBytes xsub
        moveq   #iob.smul,d0
bytes   move.l  chid(sp),a0     ; Channel
        move.l  place(sp),a1    ; base of buffer
        regsave d3/d4/d5/d7/a2/a3
        move.l  d0,d7           ; remember command to use
        moveq   #0,d0           ; no error just yet
        move.l  length+24(sp),d4; number of bytes to get/send
        move.l  count+24(sp),a2 ; place for return
        moveq   #0,d5           ; number of bytes I've got/sent
        timeout                 ; get timeout
gbloop  move.l  d4,d2           ; length still to handle
        beq.s   gbdone          ; nothing -> done
        cmpi.l  #$8000,d2       ; split up in pieces that os can cope with
        bmi.s   gbd2ok          ; can do
        move.w  #$7fff,d2       ; as much as possible in one call
gbd2ok  move.l  d7,d0           ; command
        trap    #do.io          ; do it
        ext.l   d1              ; make long
        sub.l   d1,d4           ; reduce number of bytes to go
        add.l   d1,d5           ; increase number we have done
        tst.w   d0              ; error ?
        beq.s   gbloop          ; no -> continue
gbdone  move.l  d5,(a2)         ; fill in int *got / int *sent
        subend
;
; get a string from channel
buff    equ     8               ; .l buffer
space   equ     12              ; .l max length for string
_IOGetString xsub
        move.l  chid(sp),a0     ; Channel
        regsave d3/d4/a2
        move.l  buff+12(sp),a2  ; buffer
        move.l  space+12(sp),d4 ; max length
                                ; get timeout
        move.l  a0,d3           ; Channel
        move.l  TimeOuts(pc),a1 ; base of table
        add.w   d3,d3           ; calculate ...
        add.w   d3,d3           ; ... offset
        move.w  2(a1,d3.w),d1   ; PutBack status !!
        clr.w   2(a1,d3.w)      ; no more PutBack character
        move.w  0(a1,d3.w),d3   ; TimeOut
        tst.w   d1              ; was there a PutBack character
        bne.s   gotbyt          ; yes -> use it !!
gsloop  moveq   #err.bffl,d0    ; buffer full ??
        subq.w  #1,d4           ; one less character to go
        bmi.s   gsdone          ; buffer full
        moveq   #iob.fbyt,d0    ; fetch byte
        trap    #do.io          ; do it
gotbyt  cmpi.b  #k.nl,d1        ; end of line ??
        beq.s   gsdone          ; -> done
        move.b  d1,(a2)+        ; set byte
        beq.s   gsdone          ; end of string -> done
        tst.w   d0              ; error ??
        beq.s   gsloop          ; no -> next
        subq.l  #1,a2           ; oops, stored bad byte in string
gsdone  clr.b   (a2)            ; end of string
        subend
;
; put a string from channel
number  equ     12              ; .l (int *) number of characters sent
_IOPutString xsub
        move.l  chid(sp),a0     ; Channel
        regsave d3/d4/a2
        move.l  buff+12(sp),a2  ; buffer
        timeout                 ; get timeout
        moveq   #0,d4           ; number of chars successfully sent
psloop  move.b  (a2)+,d1        ; next byte
        beq.s   pslast
        moveq   #iob.sbyt,d0    ; send byte
        trap    #do.io          ; do it
        tst.w   d0              ; error ??
        bne.s   psdone
        addq.l  #1,d4
        bra.s   psloop
pslast  moveq   #k.nl,d1        ; send newline
        moveq   #iob.sbyt,d0    ; send byte
        trap    #do.io          ; do it
        tst.w   d0              ; error ??
        bne.s   psdone
        addq.l  #1,d4
psdone  move.l  number+12(sp),a2; number of bytes sent
        move.l  d4,(a2)         ; fill in
        subend
;
; IOTest, IOFlush, IOTruncate
_IOTest xsub
        moveq   #iob.test,d0    ; test
        bra.s   simple
_IODone xsub
        moveq   #iof.chek,d0    ; flush
        bra.s   simple
_IOFlush xsub
        moveq   #iof.flsh,d0    ; flush
        bra.s   simple
_IOTruncate xsub
        moveq   #iof.trnc,d0    ; flush
simple  move.l  chid(sp),a0     ; Channel
        regsave d3
        timeout
        trap    #do.io          ; do it
        subend
;
; get current position in file
pos     equ     8               ; file position
_IOPositionGet xsub
        move.l  chid(sp),a0     ; Channel
        move.l  pos(sp),d1      ; position
        regsave d3/a2
        move.l  d1,a2           ; pos for return
        moveq   #0,d1           ; same position
        timeout
        moveq   #iof.posr,d0    ; position relative
        trap    #do.io          ; do it
        move.l  d1,(a2)         ; return current file position
        subend
;
; set new file position
_IOPositionSet xsub
        moveq   #iof.posa,d0
        bra.s   position
;
; move current file position
_IOPositionMove xsub
        moveq   #iof.posr,d0
position
        move.l  chid(sp),a0     ; Channel
        move.l  pos(sp),d1      ; position absolute or relative
        timeout
        trap    #do.io
        subend
;
        end
