/*	bsearch(3)
 *
 *	Author: Terrence Holm		   Aug. 1988
 *
 *
 *	Performs a binary search for a given <key> within a sorted
 *	table. The table contains <count> entries of size <width>
 *	and starts at <base>.
 *
 *	Entries are compared using keycmp( key, entry ), each argument
 *	is a (void *), the function returns an int < 0, = 0 or > 0
 *	according to the order of the two arguments.
 *
 *	Bsearch(3) returns a pointer to the matching entry, if found,
 *	otherwise NULL is returned.
 *
 * adjusted 4 February 1995 by Joachim Van der Auwera to be part of the
 * PROGS SMS C library
 */

#include "err_h"
#include "sms_h"
#include "array_h"

Error ARRAYSearch(void *key, void *base, Size count, Size width,
				  int (*compare)(const void *, const void *), void **ret)
{
	void *mid_point;
	int cmp;

	while (count != 0)
	{
		mid_point = (void *)((char *)base + width * (count>>1));

		cmp = compare(key, mid_point);

		if (cmp == 0)
		{
			*ret=mid_point;
			return ERR_OK;
		}

		if (cmp < 0)
			count >>= 1;
		else
		{
			base = (void *)((char *)mid_point + width);
			count = (count - 1) >> 1;
		}
	}

	return ERR_ITNF;
}
