/*	Standard library file for PROGS SMS C library */
/*
 *	Author: Terrence W. Holm		  Nov. 1988
 *	adjusted 23 November 1994, for PROGS SMS C library
 *				   by Joachim Van der Auwera for PROGS
 *
 *
 *	A prime modulus multiplicative linear congruential
 *	generator (PMMLCG), or "Lehmer generator".
 *	Implementation directly derived from the article:
 *
 *	S. K. Park and K. W. Miller
 *	Random Number Generators: Good Ones are Hard to Find
 *	CACM vol 31, #10. Oct. 1988. pp 1192-1201.
 *
 *
 *	Using the following multiplier and modulus, we obtain a
 *	generator which:
 *
 *	1)	Has a full period: 1 to 2^31 - 2.
 *	2)	Is testably "random" (see the article).
 *	3)	Has a known implementation by E. L. Schrage.
 */

#include "sms_h"

#define  A		  16807L	/*	A "good" multiplier */
#define  M	 2147483647L	/*	Modulus: 2^31 - 1	*/
#define  Q		 127773L	/*	M / A				*/
#define  R		   2836L	/*	M % A				*/

static long seed=RANDOM_INIT;		/* seed for randomizer, global to make */
									/* it less predicatble, because it now */
									/* depends on all program which use */
									/* SMSRandom */

Error SMSRandom(int *result)
{
	seed = A * (seed % Q) - R * (seed / Q);

	if (seed<0)
		seed += M;

	*result=seed;

	return ERR_OK;
}

Error SMSRandomSeed(int newseed)
{
	seed=newseed;
	if (seed<0)
		seed += M;
	return ERR_OK;
}
