/*
	Standard library file for PROGS SMS C library

	string handling library
	STRFormatPut
	STRFormatPutSafe

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, started 9 November 1994
	02.01.1995 fixed a problm with %n (was wrong and could cause crashes too)
*/

#include "sms_h"
#include "stdarg_h"
#include "str_h"
#include "ctype_h"
#include "math_h"
#include "str_h"

double atof(char *s);

#define SPC 01
#define STP 02

#define SHORT	0
#define REGULAR 1
#define LONG	2
#define INT 	0
#define FLOAT	1

static int _innum(int *ptr, int type, int len, int size, char **string, int *nchars, char *_sctab);
static int _instr(char *ptr, int type, int len, char **string, int *nchars, char *_sctab);
static char *_getccl(const unsigned char *s, char *_sctab);

Error STRFormatGet(char *str, int *count, char *format, ...)
{
	char *string=str;				/* special copy, to put it in a reg */
	char *fmt=format;				/* special copy, to put it in a reg */
	int ch; 						/* current character in format string */
	int len;						/* field width */
	int ch1;						/* next character in string */
	int *ptr;						/* pointer to the data to fill in */
	char fileended=FALSE;			/* current eof status */
	int size;						/* size of object */
	va_list argp;					/* variable argument list */
	int nchars=0;					/* number of characters already handled */
	char _sctab[256];				/* table for matching purposes */
	int _count; 					/* spare for when no count given */

	if (count==NULL) count=&_count; /* fill in when no value given */
	*count=0;						/* initialise to zero */

	MEMClear(256,_sctab);			/* initialise matching table */
	_sctab[' ']=_sctab['\t']=_sctab['\v']=_sctab['\n']=_sctab['\r']=SPC;

	va_start(argp,format);

	while(TRUE) switch (ch = *fmt++)
	{
	case '\0':
		return ERR_OK;
	case '%':
		if ((ch = *fmt++) == '%')
			goto def;				/* just a percent char, handle as such */

		if (ch == 'n')				/* filled number of characters scanned ? */
		{
			ptr=va_arg(argp,int *);
			*ptr=nchars;
			break;
		}

		/* if nothing left to scan -> eof */
		if (!*string) return ERR_EOF;

		ptr = NULL; 				/* don't fill in value */
		if (ch != '*')				/* just skip this value ?? */
			ptr=va_arg(argp,int *); /* no -> place to fill in */
		else
			ch = *fmt++;			/* just get next format descriptor char */

		len = 0;					/* maximum field width to use */
		size = REGULAR; 			/* assume regular size */
		while (isdigit(ch)) 		/* get width */
		{
			len = len*10 + ch - '0';
			ch = *fmt++;
		}
		if (!len) len=INFINIT;		/* default if none given (bit long?) */

		if (ch=='l')				/* LONG */
		{
			size = LONG;
			ch = *fmt++;
		}
		else if (ch=='h')			/* SHORT */
		{
			size = SHORT;
			ch = *fmt++;
		}
		else if (ch=='[')			/* handle a set of characters */
			fmt = _getccl((const unsigned char *)fmt, _sctab);
		if (isupper(ch))
		{
			ch = tolower(ch);
			size = LONG;
		}
		if (ch == '\0') 			/* oops, invalid end of format string */
			return ERR_ISYN;
									/* handle the actual scanning */
		if (_innum(ptr, ch, len, size, &string, &nchars, _sctab) && ptr)
			(*count)++;
		break;

	case ' ':
	case '\n':						/* handle whitespace */
	case '\t':
	case '\r':
	case '\f':
	case '\v':
		while (isspace(*string))
		{
			string++;
			nchars++;
		}
		break;

	default:						/* an exact copy if the input !!! */
	def:
		/* see if there is still a character to match */
		if (!(ch1=*string++)) return ERR_EOF;

		/* does it match */
		if (ch1!=ch)
			return ERR_ITNF;		/* oops, it doesn't */
		else
			nchars++;				/* another matching character */
		break;
	}
}

static int _innum(int *ptr, int type, int len, int size, char **str, int *nchars, char *_sctab)
{
	char *string=*str;				/* current pos in buffer */
	char numbuf[64];				/* buffer to put the number in */
	register char *np=numbuf;		/* pointer into number buffer */
	register int c; 				/* current character in string */
	register int base=10;			/* base of numbering system */
	char expseen=FALSE; 			/* have we encountered an exponent ? */
	int scale=INT;					/* type of parameter, INT or FLOAT */
	char negflg=FALSE;				/* is number negative ? */
	int ndigit=0;					/* number of digits already encountered */
	long lcval=0;					/* current value, during conversion */
	int cpos=0; 					/* character position during conversion */

	/* if we need a character or a string, then hand of to string routine */
	if (type=='c' || type=='s' || type=='[')
		return(_instr(ptr? (char *)ptr: (char *)NULL, type, len,
					  str, nchars, _sctab));

	if (type=='e' || type=='f' || type=='g')
		scale = FLOAT;
	else if (type=='o')
		base = 8;
	else if (type=='x')
		base = 16;

	/* handle whitespace in front of number */
	while (isspace(*string))
	{
		string++;
		(*nchars)++;
	}
	c=*string++;					/* current character */
	(*nchars)++;

	if (c=='-') 					/* minus sign in front of number ? */
	{
		negflg=TRUE;				/* negative number */
		*np++ = c;					/* first char in buffer should indicate minus */
		goto sign;
	}
	else if (c=='+')				/* plus sign in front of number ? */
	{
sign:	c = *string++;				/* get next character */
		(*nchars)++;				/* we have already handled one char */
		len--;						/* one char of field width handled */
	}
	for ( ; --len>=0 ; *np++ = c, c = *string++, (*nchars)++)
	{
		cpos++;

		/* handle identifier for octal or hex number */
		if (c == '0' && cpos == 1 && type == 'i')
		  base = 8;
		if (cpos==2 && type=='i' && (c=='x' || c=='X'))
		{
		  base = 16;
		  continue;
		}

		if (isdigit(c) || (base==16 && isxdigit(c) ))
		{
			ndigit++;
			if (base==10)
				lcval = ((lcval<<2) + lcval)<<1;
			else if (base==8)
				lcval <<=3;
			else
				lcval <<= 4;
			if (isdigit(c))
				lcval+= c-'0';
			else
				lcval+= toupper(c)-'A'-10;
			continue;
		}
		else if (c=='.')
		{
			if (base!=10 || scale==INT)
				break;
			ndigit++;
			continue;
		}
		else if ((c=='e'||c=='E') && !expseen)
		{
			if (base!=10 || scale==INT || ndigit==0)
				break;
			expseen=TRUE;
			*np++ = c;
			c = *string++;
			(*nchars)++;
			if (c!='+'&&c!='-'&&('0'>c||c>'9'))
				break;
		}
		else
			break;
	}
	if (negflg)
		lcval = -lcval;
	string--;
	(*nchars)--;
	if (ptr==NULL || np==numbuf || (negflg && np==numbuf+1) ) /* gene dykes */
	{
		*str=string;
		return FALSE;
	}
	*np++ = '\0';

	if (scale==FLOAT)
		*(double *)ptr = atof(numbuf);
	else switch(size)
	{                               /* now fill in the value */
	case SHORT:
		*(short *)ptr = (short)lcval;
		break;

	case REGULAR:
		*(int *)ptr = (int)lcval;
		break;

	case LONG:
		*(long *)ptr = lcval;
		break;
	}
	*str=string;
	return TRUE;
}

static int _instr(char *ptr, int type, int len, char **str, int *nchars, char *_sctab, char *_sctab)
{
	char *string=*str;				/* current pos in buffer */
	register char ch;				/* current char */
	register char *optr=ptr;		/* original address for string */
	int ignstp=0;					/* stop on whitespace char ?? */

	if (type=='c' && len==INFINIT)
		len = 1;
	if (type=='s')
	{
		/* skip whitespace */
		while (isspace(*string))
		{
			string++;
			(*nchars)++;
		}
		ignstp = SPC;
	}
	ch=*string++;					/* next character in string */
	(*nchars)++;

	if (type=='[')
		ignstp = STP;
	while (ch && (_sctab[ch]&ignstp)==0)
	{
		if (ptr)					/* fill in string if necessary */
			*ptr++ = ch;
		if (--len <= 0) 			/* check if still in field */
			break;
		ch=*string++;				/* next character in string */
		(*nchars)++;				/* one more character handled */
	}
	if (len > 0)
	{
		string--;
		nchars--;
	}
	if (ptr!=optr)
	{
		if (type!='c')
			*ptr++ = '\0';
		*str=string;				/* update current position in string */
		return TRUE;
	}
	return FALSE;
}

static char *_getccl(const unsigned char *s, char *_sctab)
{
	register c, t;

	t = 0;
	if (*s == '^')
	{
		t++;
		s++;
	}
	for (c = 0; c < (sizeof _sctab / sizeof _sctab[0]); c++)
		if (t)
			_sctab[c] &= ~STP;
		else
			_sctab[c] |= STP;
	if ((c = *s) == ']' || c == '-')
	{
		/* first char is special */
		if (t)
			_sctab[c] |= STP;
		else
			_sctab[c] &= ~STP;
		s++;
	}
	while ((c = *s++) != ']')
	{
		if (c==0)
			return((char *)--s);
		else if (c == '-' && *s != ']' && s[-2] < *s)
		{
			for (c = s[-2] + 1; c < *s; c++)
				if (t)
					_sctab[c] |= STP;
				else
					_sctab[c] &= ~STP;
		} else if (t)
			_sctab[c] |= STP;
		else
			_sctab[c] &= ~STP;
	}
	return((char *)s);
}

#define MSIGN	1					/* mantissa has negative sign */
#define ESIGN	2					/* exponent has negative sign */
#define DECPT	4					/* decimal point encountered */

int _mul10add(double *valuep,int digit);
double _adjust(double *valuep,int decexp,int negflag);

double atof(char *s)
{
	int flags = 0;
	int decexp = 0; 				/* decimal exponent */
	double value = 0;				/* accumulated value - actually */
									/* a 64-bit integer */


	while (isspace(*s)) 			/* skip leading white space */
		++s;

	if(*s == '+')
		++s;
	else if(*s == '-')
	{
		++s;
		flags |= MSIGN; 			/* mantissa is negative */
	}

	for(; ; ++s)
	{
		if( isdigit(*s))
		{
			if (_mul10add(&value, *s - '0'))
				++decexp;
			if (flags & DECPT)
				--decexp;
		}
		else if(*s == '.')
		{
			flags |= DECPT;
		}
		else
			break;
	}

	if(*s == 'e' || *s == 'E')
	{
		int eacc = 0;				/* exponent accumulator */

		++s;

		if (*s == '+')
			++s;
		else if (*s == '-')
		{
			++s;
			flags |= ESIGN; 		/* decimal exponent is negative */
		}

		while (isdigit(*s))
		{
			if(eacc < 1000)
				eacc = eacc * 10 + (*s - '0');
			++s;
		}

		if (flags & ESIGN)
			decexp -= eacc;
		else
			decexp += eacc;
	}

	if (decexp > 350)				/* outrageously large */
		decexp = 350;
	else if (decexp < -350) 		/* outrageously small */
		decexp = -350;

	return _adjust(&value, decexp, flags & MSIGN);
}
