; DEV library for the PROGS SMS C library
;   DEVDataGet          get current data directory
;   DEVDataSet          set new data directory
;   DEVDataUp           go up one level on data directory
;   DEVDataDown         move data directory down to given subdirectory
;   DEVProgramGet       get current program directory
;   DEVProgramSet       set new program directory
;   DEVProgramUp        go up one level on program directory
;   DEVProgramDown      move program directory down to given subdirectory
;   DEVDestinationGet   get current destination directory
;   DEVDestinationSet   set new destination directory
; written by Joachim Van der Auwera for PROGS
; (c) 1994 by PROGS, Professional & Graphical Software, all rights reserved
; started 26 October 1994
        section text
        include win1_inc_qdos_sms
        include win1_inc_keys_err
        include win1_inc_keys_sys
        include win1_inc_aid_mac
;
; routines to get the current system default
string  equ     4               ; string to store result
_DEVDataGet xsub
        move.w  #sys_datd,d0    ; offset for pointer to data directory
        bra.s   devget
_DEVProgramGet xsub
        move.w  #sys_prgd,d0    ; offset for pointer to program directory
        bra.s   devget
_DEVDestinationGet xsub
        move.w  #sys_dstd,d0    ; offset for pointer to destination device
devget  move.l  string(sp),a1   ; place to fill in return value
        regsave d2/d3
        move.w  d0,d3           ; keep offset !!
        moveq   #sms.info,d0    ; get system info
        trap    #do.sms2        ; do it
        move.l  (a0,d3.w),a0    ; string with requested device
        move.w  (a0)+,d0        ; length of string
        bra.s   getel
getl    move.b  (a0)+,(a1)+     ; copy a byte
getel   dbf     d0,getl         ; another one ??
        clr.b   (a1)+           ; end of string marker
        moveq   #0,d0           ; no error
        subend
;
; routines to set the current system default
_DEVDataSet xsub
        move.w  #sys_datd,d0    ; offset for pointer to data directory
        bra.s   devset
_DEVProgramSet xsub
        move.w  #sys_prgd,d0    ; offset for pointer to program directory
        bra.s   devset
_DEVDestinationSet xsub
        move.w  #sys_dstd,d0    ; offset for pointer to destination device
devset  move.l  string(sp),a1   ; place to fill in return value
        regsave d2/d3/a2/a3
        move.w  d0,d3           ; keep offset !!
        moveq   #sms.info,d0    ; get system info
        trap    #do.sms2        ; do it
        move.l  (a0,d3.w),a2    ; base for string with requested device
        move.l  a1,-(sp)        ; string
        xjsr    _strlen         ; calculate length of it (d0)
        move.l  (sp)+,a1        ; clean up stack, refill a1 with string base
        cmpi.w  #34,d0          ; is length valid ??
        bgt.s   setbffl
        move.l  a2,a3           ; keep for adding underscore
        move.w  d0,(a2)+        ; fill in length of string
        bra.s   setel
setl    move.b  (a1)+,(a2)+     ; copy a byte
setel   dbf     d0,setl         ; another one ??
        moveq   #0,d0           ; no error
        cmpi.w  #sys_dstd,d3    ; are we seeting the destination default ??
        beq.s   setok           ; yes -> done
        move.w  (a3),d1         ; current length of string
        cmpi.b  #'_',1(a3,d1.w) ; end in underscore ??
        beq.s   setok           ; yes -> done
        cmpi.w  #34,d1          ; can we add a character ??
        beq.s   setok           ; no -> done
        addq.w  #1,(a3)         ; make string one longer
        move.b  #'_',2(a3,d1.w) ; add underscore at end
        bra.s   setok           ; and return
setbffl moveq   #err.bffl,d0    ; buffer full
setok   subend                  ; restore and return
;
; routines to move up one level on the current system default
_DEVDataUp xsub
        move.w  #sys_datd,d0    ; offset for pointer to data directory
        bra.s   devup
_DEVProgramUp xsub
        move.w  #sys_prgd,d0    ; offset for pointer to program directory
devup   regsave d2/d3
        move.w  d0,d3           ; keep offset !!
        moveq   #sms.info,d0    ; get system info
        trap    #do.sms2        ; do it
        move.l  (a0,d3.w),a0    ; base for string with requested device
        move.w  (a0),d3         ; length of current version
uplp    cmpi.b  #'_',0(a0,d3.w) ; see where previous level ends
        dbeq    d3,uplp         ; not found -> search on
        bne.s   upend           ; this was the only level !!
                                ; this relies on code('_') being < 34 !!
        subq.w  #1,d3           ; make length correct
        move.w  d3,(a0)         ; fill in new length
upend   subend                  ; restore and return
;
; routines to set the current system default
_DEVDataDown xsub
        move.w  #sys_datd,d0    ; offset for pointer to data directory
        bra.s   devdown
_DEVProgramDown xsub
        move.w  #sys_prgd,d0    ; offset for pointer to program directory
devdown move.l  string(sp),a1   ; place to fill in return value
        regsave d2/d3/a2/a3
        move.w  d0,d3           ; keep offset !!
        moveq   #sms.info,d0    ; get system info
        trap    #do.sms2        ; do it
        move.l  (a0,d3.w),a3    ; base for string with requested device
        move.l  a1,-(sp)        ; string
        xjsr    _strlen         ; calculate length of it (d0)
        move.l  (sp)+,a1        ; clean up stack, refill a1 with string base
        move.w  (a3),d1         ; current length of device
        lea     2(a3,d1.w),a2   ; address of end of string
        add.w   d0,d1           ; added length by moving down
        cmpi.w  #34,d1          ; is length valid ??
        bgt.s   downbffl
        move.w  d1,(a3)         ; fill in new length of string
        bra.s   downel
downl   move.b  (a1)+,(a2)+     ; copy a byte
downel  dbf     d0,downl        ; another one ??
        moveq   #0,d0           ; no error
        move.w  (a3),d1         ; current length of string
        cmpi.b  #'_',1(a3,d1.w) ; end in underscore ??
        beq.s   downok          ; yes -> done
        cmpi.w  #34,d1          ; can we add a character ??
        beq.s   downok          ; no -> done
        addq.w  #1,(a3)         ; make string one longer
        move.b  #'_',2(a3,d1.w) ; add underscore at end
        bra.s   downok          ; and return
downbffl
        moveq   #err.bffl,d0    ; buffer full
downok  subend                  ; restore and return
;
        end
