/*
	Standard library file for PROGS SMS C library

	error handling library
	ERRMessageGet

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, 3 December 1995
	10.01.1996 fixed problem on SMSQ/E when error code which is passed is
			   positive. The OS does not return an error and also no message
			   pointer. Before, this could result in crashes.
	10.04.1996 added message for new ERR_ACCD, ERR_NOMS overlaps with ERR_RWF
*/

#include "err_h"
#include "str_h"

char *messages[] = {
	"no error",
	"operation not complete",
	"invalid job id",
	"insufficient memory",
	"parameter outside permitted range",
	"buffer full",
	"invalid channel id",
	"file or device not found",
	"item not found",
	"file already exists",
	"file or device in use",
	"end of file",
	"drive full",
	"invalid file, device or thing name",
	"transmission error",
	"format drive failed",
	"invalid parameter",
	"file system medium check failed",
	"invalid expression",
	"arithmetic overflow",
	"operation not implemented",
	"read only permitted",
	"invalid syntax",
	"read or write failed",
	"access denied"
};

typedef struct {
	short len;
	char str[2];
} Message;

Error _MessagePtr(Error err, Message **mess);

Error ERRMessageGet(Error error, char *message)
{
	Error err;
	Message *mess;

	if (error<0 && ( !_MessagePtr(error,&mess) ||
		( (mess=(Message *)(error&0x7fffffff)) && !(error&0x40000000)) ) )
	{
		if (mess->len>ERR_MAXLEN-1)
			STRCopy("unknown error",message);
		else
		{
			MEMCopy(ERR_MAXLEN,mess->str,message);
			message[mess->len]='\0';
		}
	}
	else
	{
		if (error<ERR_RWF || error>0)
			STRCopy("unknown error",message);
		else
			STRCopy(messages[-error],message);
	}
	return ERR_OK;
}
