; JOB library for the PROGS SMS C library
;   JOBMe               get Job identifier for this job
;   JOBInfo             get information about the job (name, priority, suspension)
;   JOBTree             scan through the job tree
;   JOBCreate           create a job (allocate the memory for it)
;   JOBRemove           remove job (only for inactive jobs)
;   JOBZap              force remove job (always)
;   JOBSuspend          suspend a job for a certain time (number of ticks)
;   JOBUnSuspend        unsuspend job
;   JOBActivate         activate a job
;   JOBActivateWait     activate a job, but wait for the result
;   JOBPriority         set the priority for a job
; written by Joachim Van der Auwera for PROGS
; (c) 1994 by PROGS, Professional & Graphical Software, all rights reserved
; started 14 December 1994
; 04.12.1997 JOBInfo now copies the maximum name length always (to allow
;            checking hidden info like the version number
        section text
        include win1_inc_qdos_sms
        include win1_inc_keys_err
        include win1_inc_aid_mac
;
; get identifier for this job
job     equ     4               ; .l place to fill in job id
_JOBMe xsub
        move.l  job(sp),a1      ; place for return
        regsave d2
        moveq   #sms.info,d0    ; get info
        trap    #do.sms2
        move.l  d1,(a1)         ; fill in return
        subend
;
; get information about the job (name, priority, suspension)
name    equ     8               ; .l place for jobname
susp    equ     12              ; .l place for suspend status (.b)
prior   equ     16              ; .l place for priority (.b)
ISP     equ     8
_JOBInfo xsub
        regsave d2/d3
        move.l  ISP+job(sp),d2  ; job at top of tree
        move.l  d2,d1           ; info on this job
        moveq   #sms.injb,d0    ; get info on job
        trap    #do.sms2
        tst.l   d0              ; success ??
        bne.s   idone
        move.l  ISP+name(sp),d0 ; fill in name
        beq.s   iname
        move.l  d0,a1           ; place for parameter
        clr.b   (a1)            ; assume no name
        lea     6(a0),a0        ; base for standard identifier
        cmpi.w  #$4afb,(a0)+    ; standard job header ??
        bne.s   iname           ; no -> no name
        move.w  (a0)+,d0        ; name length
        cmpi.w  #63,d0          ; see if not too long
        bmi.s   inamo
        moveq   #63,d0
inamo   moveq   #63,d1          ; number of bytes to copy
inaml   move.b  (a0)+,(a1)+     ; copy a byte
        dbf     d1,inaml        ; another ?
        move.l  ISP+name(sp),a1 ; fill in name
        clr.b   0(a1,d0.w)      ; set end of string marker
iname   move.l  ISP+susp(sp),d0 ; fill in susp
        beq.s   isusp
        move.l  d0,a1           ; place for parameter
        clr.b   (a1)            ; assume FALSE
        tst.l   d3              ; is it ??
        bpl.s   isusp           ; no -> ok
        move.b  #$ff,(a1)       ; set TRUE
isusp   move.l  ISP+prior(sp),d0 ; fill in prior
        beq.s   iprior
        move.l  d0,a1           ; place for parameter
        move.b  d3,(a1)
iprior  moveq   #0,d0
idone   subend
;
; scan through the job tree
scan    equ     8               ; .l ptr to current job (adjusted)
TSP     equ     8               ; space occupied by regsave
_JOBTree xsub
        regsave d2/d3
        move.l  TSP+job(sp),d2  ; job at top of tree
        move.l  TSP+scan(sp),a1 ; current job
        move.l  (a1),d1         ; fill in
        moveq   #sms.injb,d0    ; get info on job
        trap    #do.sms2
        move.l  TSP+scan(sp),a1 ; current job
        move.l  d1,(a1)         ; fill in return
        subend
;
; activate job
prio    equ     8               ; .b priority
ASP     equ     12
_JOBPriority xsub
        regsave d2/d3/a3
        moveq   #sms.spjb,d0    ; set priority of job
        bra.s   priori
_JOBActivate xsub
        regsave d2/d3/a3
        moveq   #0,d3           ; don't wait for result
        bra.s   activ
_JOBActivateWait xsub
        regsave d2/d3/a3
        moveq   #-1,d3          ; don't wait for result
activ   moveq   #sms.acjb,d0    ; activate job
priori  move.l  ASP+job(sp),d1  ; job id
        move.b  ASP+1+prio(sp),d2   ; priority
        trap    #do.sms2
        subend
;
; create a job
size    equ     8               ; .l cde length
data    equ     12              ; .l dataspace
start   equ     16              ; .l start address
id      equ     20              ; .l place for job id
base    equ     24              ; .l place for base of job
CSP     equ     4
_JOBCreate xsub
        regsave d3
        move.l  CSP+job(sp),d1  ; owner job
        move.l  CSP+size(sp),d2 ; code length
        move.l  CSP+data(sp),d3 ; dataspace length
        move.l  CSP+start(sp),a1; start of code
        moveq   #sms.crjb,d0    ; create job
        trap    #do.sms2        ; do it
        tst.l   d0
        bmi.s   cdone
        move.l  CSP+id(sp),d0   ; place for result
        beq.s   cid
        move.l  d0,a1
        move.l  d1,(a1)         ; new job id
cid     move.l  CSP+base(sp),d0 ; place for result
        beq.s   cbase
        move.l  d0,a1
        move.l  a0,(a1)         ; new job base
cbase   moveq   #0,d0           ; no error
cdone   subend
;
; force remove a job (and all jobs, memory and things owned by it)
err     equ     8               ; .l error
ZSP     equ     16
_JOBRemove xsub
        moveq   #sms.rmjb,d0    ; remove job
        bra.s   remv
_JOBZap xsub
        moveq   #sms.frjb,d0    ; force remove job
remv    regsave d2/d3/a2/a3
        move.l  ZSP+job(sp),d1
        move.l  ZSP+err(sp),d3
        trap    #do.sms2        ; do it
        subend
;
; unsuspend a job
_JOBUnSuspend xsub
        move.l  job(sp),d1      ; job id
        moveq   #sms.usjb,d0    ; unsuspend job
        trap    #do.sms2        ; do it
        subend
;
; suspend a job
timeout equ     8               ; .w timeout
flag    equ     10              l .l address of flag byte
SSP     equ     4
_JOBSuspend xsub
        regsave d3
        move.l  SSP+job(sp),d1
        move.w  SSP+timeout(sp),d3
        move.l  SSP+flag(sp),a1
        moveq   #sms.ssjb,d0    ; suspend a job
        trap    #do.sms2        ; do it
        subend
;
        end
