/*
	Standard library file for PROGS SMS C library

	external module handling library
	MODULELoadFor

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, started 2 December 1994
*/

#include "io_h"
#include "str_h"
#include "module_h"

/* external definition, routine in DLL Manager */
void *MODULELink(void *base);

Error MODULELoadFor(char *name, char *path, Job job, char *id, void **init)
{
	Error err;
	Size length, data;
	Channel chid;
	FileType ft;
	char *base;

	*init=NULL;

	/* open the file and test for the filetype */
	err=IOOpenPath(name,OPEN_READ,path,&chid);
	if (err) return err;
	err=IOFileTypeGet(chid,&ft);
	if (err) return err;
	if (ft!=FILETYPE_EXTERNAL_MODULE) return ERR_ISYN;

	/* allocate memory for the module */
	err=IOLength(chid,&length);
	if (err) { IOClose(chid); return err; }
	err=IODataSpaceGet(chid,&data);
	if (err) { IOClose(chid); return err; }
	if (data<0) data=0; 			/* make sure dataspace is positive */
	err=MEMAllocateFor(length+data,job,&base);
	if (err) { IOClose(chid); return err; }

	/* load the module */
	err=IOFileLoad(chid,length,base);
	IOClose(chid);
	if (err)
	{
		MEMRelease(base);
		return err;
	}

	/* check for a match in the type (cfr identifier) */
	if (!STRSameCD(base,id)) return ERR_ISYN;

	*init=MODULELink(base + even(strlen(id)+1));
	if (*init==NULL) return ERR_ITNF;
				else return ERR_OK;
}
