; THING library for the PROGS SMS C library
;   THINGLink           link in a new thing
;   THINGRemove         (gently) remove a thing
;   THINGUse            use a thing
;   THINGUseFor         use a thing (giving job id and timeout)
;   THINGFree           free a thing
;   THINGFreeFor        free a thing for another job
;   THINGZap            force remove thing
;   THINGNext           cycle through thing list
;   THINGUserNext       cycle through thing user list
;   THINGCallIn         call a c parameter thing extension
;   THINGCallBlockIn    call a parameter block thing extension
;   THINGFreeFull       thing free routine, all possible params/returns
;   THINGUseFull        thing use routine, all possible params/returns
; written by Joachim Van der Auwera for PROGS
; (c) 1994 by PROGS, Professional & Graphical Software, all rights reserved
; started 4 December 1994
; 11.11.1995 CallIn routines returned wrong error when extension not found
;            should be nimp (not itnf)
        section text
        include win1_inc_qdos_sms
        include win1_inc_keys_err
        include win1_inc_keys_sys
        include win1_inc_keys_thg
        include win1_inc_aid_mac
;
        xref    THINGcall
;
; link in a new thing
thing   equ     4               ; .l address of thing to link in
_THINGLink xsub
        move.l  thing(sp),a1    ; thing to link in
        moveq   #sms.lthg,d0    ; link in new thing
        bsr     THINGcall
        subend
;
; remove a thing
for     equ     8               ; .l job to free the thing for
_THINGRemove xsub
        moveq   #sms.rthg,d0    ; remove thing
        bra.s   thrzf
_THINGFree xsub
        moveq   #sms.fthg,d0    ; free thing
        moveq   #sms.myjb,d1    ; for myself
        bra.s   thrzf
_THINGFreeFor xsub
        moveq   #sms.fthg,d0    ; remove thing
        move.l  for(sp),d1      ; owner job
        bra.s   thrzf
_THINGZap xsub
        moveq   #sms.zthg,d0    ; remove thing
thrzf   move.l  thing(sp),a1    ; name of thing
        regsave d2/d3/a2
        getssp  66              ; need some stack space for thingname
        lea     2(sp),a0        ; place to copy string to
        moveq   #63,d3          ; max length of string
        moveq   #-1,d2          ; counter of string length
copystr addq.w  #1,d2           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d3,copystr
        move.w  d2,(sp)         ; set length of string
        move.l  sp,a0           ; base of string with thingname
        bsr     THINGcall       ; do it
        subend
;
; thing free with all possible parameters
pd2     equ     12              ; value for d2
pd3     equ     16              ; value for d3
pa1     equ     20              ; value for a1
pa2     equ     24              ; value for a2
rd2     equ     28              ; place for return in d2
ra2     equ     32              ; place for return in a2
TFSP    equ     66+12
_THINGFreeFull xsub
        regsave d2/d3/a2
        getssp  66
        lea     2(sp),a0        ; place to copy string to
        moveq   #63,d3          ; max length of string
        moveq   #-1,d2          ; counter of string length
coprstr addq.w  #1,d2           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d3,coprstr
        move.w  d2,(sp)         ; set length of string
        move.l  sp,a0           ; base of string with thingname

        moveq   #sms.fthg,d0    ; free thing
        move.l  TFSP+for(sp),d1 ; owner job
        move.l  TFSP+pd2(sp),d2
        move.l  TFSP+pd3(sp),d3
        move.l  TFSP+pa1(sp),a1
        move.l  TFSP+pa2(sp),a2
        bsr     THINGcall       ; do it
        move.l  TFSP+rd2(sp),d1
        beq.s   tfnrd2
        move.l  d1,a1
        move.l  d2,(a1)
tfnrd2  move.l  TFSP+ra2(sp),d1
        beq.s   tfnra2
        move.l  d1,a1
        move.l  a2,(a1)
tfnra2  subend
;
; use a thing
exte    equ     8               ; .l extension name
ret     equ     12              ; .l place for addr of thing
vers    equ     16              ; .l place for version of thing
UOFF    equ     12+66
_THINGUse xsub
        move.l  thing(sp),a1    ; name of thing to use
        regsave d2/d3/a2
        getssp  66              ; need some stack space for thingname
        lea     2(sp),a0        ; place to copy string to
        moveq   #63,d3          ; max length of string
        moveq   #-1,d2          ; counter of string length
copustr addq.w  #1,d2           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d3,copustr
        move.w  d2,(sp)         ; set length of string
        move.l  sp,a0           ; base of string with thingname
        move.l  UOFF+exte(sp),d2; parameter
        moveq   #sms.myjb,d1    ; for myself
        moveq   #-1,d3          ; timeout - forever
        moveq   #sms.uthg,d0    ; use thing
        bsr     THINGcall
        bne.s   udone
        move.l  UOFF+ret(sp),a0 ; place for thing
        move.l  a1,(a0)         ; fill in
        move.l  UOFF+vers(sp),d0; place for version
        beq.s   udone
        move.l  d0,a0
        move.l  d3,(a0)         ; fill in
        moveq   #0,d0
udone   subend
;
; use a thing, with job and timeout
jb      equ     12              ; .l job id
to      equ     16              ; .l timeout
fret    equ     18              ; .l place for addr of thing
fver    equ     22              ; .l place for version of thing
_THINGUseFor xsub
        move.l  thing(sp),a1    ; name of thing to use
        regsave d2/d3/a2
        getssp  66              ; need some stack space for thingname
        lea     2(sp),a0        ; place to copy string to
        moveq   #63,d3          ; max length of string
        moveq   #-1,d2          ; counter of string length
copfstr addq.w  #1,d2           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d3,copfstr
        move.w  d2,(sp)         ; set length of string
        move.l  sp,a0           ; base of string with thingname
        move.l  UOFF+exte(sp),d2; parameter
        move.l  UOFF+jb(sp),d1  ; job id
        move.w  UOFF+to(sp),d3  ; timeout
        moveq   #sms.uthg,d0    ; use thing
        bsr     THINGcall
        bne.s   fdone
        move.l  UOFF+fret(sp),a0; place for thing
        move.l  a1,(a0)         ; fill in
        move.l  UOFF+fver(sp),d0; place for version
        beq.s   fdone
        move.l  d0,a0
        move.l  d3,(a0)         ; fill in
        moveq   #0,d0
fdone   subend
;
; use a thing, with job and timeout
upara2  equ     18              ; .l parameter for a2
uretd2  equ     22              ; .l place for result in d2
uret    equ     26              ; .l place for addr of thing
uver    equ     30              ; .l place for version of thing
_THINGUseFull xsub
        move.l  thing(sp),a1    ; name of thing to use
        regsave d2/d3/a2
        getssp  66              ; need some stack space for thingname
        lea     2(sp),a0        ; place to copy string to
        moveq   #63,d3          ; max length of string
        moveq   #-1,d2          ; counter of string length
coplstr addq.w  #1,d2           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d3,coplstr
        move.w  d2,(sp)         ; set length of string
        move.l  sp,a0           ; base of string with thingname
        move.l  UOFF+exte(sp),d2; parameter
        move.l  UOFF+jb(sp),d1  ; job id
        move.w  UOFF+to(sp),d3  ; timeout
        move.l  UOFF+upara2(sp),a2
        moveq   #sms.uthg,d0    ; use thing
        bsr     THINGcall
        move.l  UOFF+uretd2(sp),d1
        beq.s   funrd2
        move.l  d1,a0
        move.l  d2,(a0)         ; fill in
funrd2  move.l  UOFF+uret(sp),d1; place for thing
        beq.s   funt
        move.l  d1,a0
        move.l  a1,(a0)         ; fill in
funt    move.l  UOFF+uver(sp),d1; place for version
        beq.s   ldone
        move.l  d1,a0
        move.l  d3,(a0)         ; fill in
ldone   subend
;
; get next thing
NOFF    equ     66
_THINGNext xsub
        move.l  thing(sp),a1    ; name of thing to use
        getssp  66              ; need some stack space for thingname
        sub.l   a0,a0           ; assume start at beginning
        tst.b   (a1)            ; is this a null string ??
        beq.s   nextdo
        lea     2(sp),a0        ; place to copy string to
        moveq   #63,d0          ; max length of string
        moveq   #-1,d1          ; counter of string length
copnstr addq.w  #1,d1           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d0,copnstr
        move.w  d1,(sp)         ; set length of string
        move.l  sp,a0           ; base of string with thingname
nextdo  moveq   #sms.nthg,d0    ; use thing
        bsr     THINGcall       ; do it
        bne.s   ndone
                                ; now fill in the return value !!
        move.l  NOFF+thing(sp),a2
        clr.b   (a2)            ; assume no return string
        move.l  a1,d0           ; is there a return thing ??
        beq.s   ndone           ; no -> done
        lea     th_name(a1),a1  ; start of thingname
        move.w  (a1)+,d0        ; length of thingname
        cmpi.w  #63,d0          ; max length for return string
        bmi.s   nlok
        moveq   #63,d0          ; only set the max length
nlok    subq.w  #1,d0           ; adjust for dbf
        bmi.s   ndone           ; noname thing ??
nclp    move.b  (a1)+,(a0)+     ; copy a byte
        dbf     d0,nclp         ; another one ??
        clr.b   (a0)+           ; eos marker
        moveq   #0,d0           ; no error
ndone   subend
;
; get next thing user
count   equ     8               ; .l place for counter to handle cycle
job     equ     12              ; .l place for job id
NUOFF   equ     74
_THINGUserNext xsub
        move.l  thing(sp),a1    ; name of thing to use
        regsave d2/a2
        getssp  66              ; need some stack space for thingname
        lea     2(sp),a0        ; place to copy string to
        moveq   #63,d0          ; max length of string
        moveq   #-1,d1          ; counter of string length
copsstr addq.w  #1,d1           ; string is one longer
        move.b  (a1)+,(a0)+     ; copy another byte
        dbeq    d0,copsstr
        move.w  d1,(sp)         ; set length of string
        move.l  sp,a0           ; base of string with thingname
        move.l  NUOFF+count(sp),a1
        move.l  (a1),a1         ; counter, NULL for init
        moveq   #sms.nthu,d0    ; use thing
        bsr     THINGcall       ; do it
        bne.s   sdone
                                ; now fill in the return values !!
        move.l  NUOFF+count(sp),a0
        move.l  a1,(a0)         ; fill in new counter
        move.l  NUOFF+job(sp),a0
        move.l  d2,(a0)         ; fill in job id of next owner
sdone   subend
;
; call a c parameter thing extension
pcount   equ    12              ; count of parameters, rest follows
thh_next equ    $08             ; offset of next thing in list
thh_exid equ    $0c             ; extra ID
thh_code equ    $18             ; entry point of extension code
_THINGCallIn xsub
        move.l  thing(sp),a0
        move.l  exte(sp),d1
        lea     pcount(sp),a1   ; start of parameters
        regsave d2-d7/a2-a5
        move.l  (a1)+,d7        ; count of parameters
        move.l  d7,d2           ; need copy of count
        lsl.w   #2,d7           ; number of bytes
        beq.s   none            ; nothing to copy
        adda.w  d7,a1           ; last parameter
        subq.w  #1,d2           ; adjust for dbf
copy    move.l  -(a1),-(sp)     ; copy a parameter
        dbf     d2,copy         ; another ??
none    moveq   #err.nimp,d0    ; assume not found
pfcall  move.l  a0,d0           ; is the thing initialised ??
        beq.s   exit            ; no -> ..
next    cmp.l   thh_exid(a0),d1 ; is this the extension we need ?
        beq.s   found           ; yes -> ..
        tst.l   thh_next(a0)    ; is there another one ?
        beq.s   exit            ; no -> ..
        add.l   thh_next(a0),a0 ; ptr to next thing in list
        bra.s   next            ; take next
found   lea     thh_code(a0),a0 ; ptr to code
        jsr     (a0)            ; call extension thing
exit    add.l   d7,sp           ; restore stack
        subend
;
; call a parameter block thing extension
block   equ     12              ; .l pointer to parameter block
_THINGCallBlockIn xsub
        move.l  thing(sp),a0    ; thing to use
        move.l  exte(sp),d1     ; extension name
        move.l  block(sp),a1    ; parameter
        regsave d2-d7/a2-a5
        moveq   #err.nimp,d0    ; assume not found
pfcall_ move.l  a0,d0           ; is the thing initialised ??
        beq.s   exit2           ; no -> ..
next_   cmp.l   thh_exid(a0),d1 ; is this the extension we need ?
        beq.s   found_          ; yes -> ..
        tst.l   thh_next(a0)    ; is there another one ?
        beq.s   exit2           ; no -> ..
        add.l   thh_next(a0),a0 ; ptr to next thing in list
        bra.s   next_           ; take next
found_  lea     thh_code(a0),a0 ; ptr to code
        jsr     (a0)            ; call extension thing
exit2   subend
;
        end
