/*
	Standard library file for PROGS SMS C library

	window handling library
	WINGet
	WINOpen
	WINOpenFull
	WINAreaAllocate
	WINAreaRelease

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, started 4 January 1995
	12.11.1995 WINAreaRelase mistakingly did not set the return value
*/

#include "win_h"
#include "str_h"
#include "mem_h"

Error WINGet(Window id, char *ret)
{
	Error err;
	err=WINCursorEnable(id);
	if (err) return err;
	err=IOGet(id,ret);
	if (err) return err;
	return WINCursorDisable(id);
}

Error _IOOpen(char *name, OpenKey key, Channel *id);

Error WINOpen(short xsiz, short ysiz, short xpos, short ypos, Window *id)
{
	char name[40];
	Error err;

	err=STRFormatPutSafe(name,40,"con_%dx%da%dx%d",xsiz,ysiz,xpos,ypos);
	if (err) return err;
	return _IOOpen(name,0,id);
}

Error WINOpenFull(WindowDescriptor *descriptor, Window *id)
{
	char name[40];
	Error err;

	/* open window */
	err=STRFormatPutSafe(name,40,"con_%dx%da%dx%d",
		descriptor->xsiz,descriptor->ysiz,descriptor->xpos,descriptor->ypos);
	if (err) return err;
	err=_IOOpen(name,0,id);
	if (err) return err;

	/* set outline and shadow */
	err=WINOutline(*id, descriptor->xshadow, descriptor->yshadow,
						descriptor->xsiz, descriptor->ysiz,
						descriptor->xpos, descriptor->ypos);
	if (err) return err;

	/* set border */
	err=WINBorder(*id,descriptor->border_width,descriptor->border_colour);
	if (err) return err;

	/* set paper and ink */
	err=WINPaper(*id,descriptor->paper_colour);
	if (err) return err;
	err=WINStrip(*id,descriptor->paper_colour);
	if (err) return err;
	err=WINInk(*id,descriptor->draw_colour);
	if (err) return err;

	/* clear window */
	return WINClear(*id);
}

Error WINAreaAllocate(short xsiz, short ysiz, WindowMode mode,
					  WindowArea **area)
{
	Size datalength;
	short linc;
	Error err;

	/* calculate the size of the data */
	switch (mode)
	{
	case WINDOWMODE_2:
		linc=even((xsiz+7)/8);
		datalength=linc*ysiz;
		break;
	case WINDOWMODE_M4:
	case WINDOWMODE_M8:
		linc=even((xsiz+3)/4);
		datalength=linc*ysiz;
		break;
	}

	err=MEMAllocate(sizeof(WindowArea)+datalength,area);
	if (err) return err;

	(*area)->flag=WINDOWAREA_FLAG;
	(*area)->xsiz=xsiz;
	(*area)->ysiz=ysiz;
	(*area)->line_increment=linc;
	(*area)->mode=mode;
	return ERR_OK;
}

Error WINAreaRelease(WindowArea *area)
{
	return MEMRelease(area);
}
