/*	Standard library file for PROGS SMS C library */

#include "math_h"

/* use Abramowitz & Stegun formula 4.3.65
 * S E Peterson 3/92 */

static double series (double x)
{
   static double coef[10] = 	{
			   1.0, 						/* 1/1!  */
			  -1.6666666666666666667E-1,	/* 1/3!  */ 
			   8.3333333333333333333e-3,	/* 1/5!  */
			  -1.9841269841269841270e-4,	/*	etc. */
			   2.7557319223985890653e-6,
			  -2.5052108385441718775e-8, 
			   1.6059043836821614599e-10,
			  -7.6471637318198164759e-13,
			   2.8114572543455207632e-15, 
			  -8.2206352466243297170e-18 
			};
   double x2, ans;
   int i;

   x2 = x * x;
   ans = coef[9]; 
   for (i = 8; i >= 0; i--)
	  ans = x2 * ans + coef[i];
   return ((ans * x));
}

double sin(double x)
{
	double twopi = M_2PI;
	double theta, ans;

	theta = fmod(x, twopi);

	if (theta > M_PI){
		theta  -= M_PI;
		ans = -series(theta);
	}
	else if (theta < -M_PI){
		theta += M_PI;
		ans = -series(theta);
	}
	else 
		ans = series(theta);
	return (ans);
}
