#ifndef _SYSLIB_MEM_H
#define _SYSLIB_MEM_H

/*
	Standard header file for PROGS SMS C library

	Memory handling library

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera, 6 October 1994
*/

#include <sms.h>

Error MEMAllocate(Size size, void **base);
Error MEMAllocateFor(Size size, Job id, void **base);
Error MEMRelease(void *base);
Error MEMWhenZapped(void *base, void (*ZapRoutine)(void *base), char *ZapAddress);

typedef long Chunks;	/* external representation of Chunks information */
						/* Zapping the same Chunks twice is considered */
						/* dangerous and can crash the system !! */
						/* same goes for the other Release routines !! */

Error MEMUserInit(Size BlockSize, int BlockingFactor, Chunks *id);
Error MEMUserAllocate(Chunks id, void **base);
Error MEMUserRelease(Chunks id, void *base);
Error MEMUserZap(Chunks id);
Error MEMUserWhenZapped(Chunks id, void (*ZapRoutine)(void *base), char *ZapAddress);

typedef long Heap;		/* external representation of a memory Heap */
Error MEMHeapInit(Size BlockSize, Heap *id);
Error MEMHeapAllocate(Heap id, Size size, void **base);
Error MEMHeapRelease(Heap id, void *base);
Error MEMHeapZap(Heap id);
Error MEMHeapWhenZapped(Heap id, void (*ZapRoutine)(void *base), char *ZapAddress);

Error MEMFree(Size *result);
Error MEMLargest(Size *result);

Error MEMCopy(Size size, void *from, void *to);
Error MEMClear(Size size, void *base);
int MEMSame(Size size, void *area1, void *area2);
Error MEMDuplicate(Size size, void *area, void **base);
Error MEMSearch(char byte, Size leng, void *base, void **at);
Error MEMSwap(Size size, void *base1, void *base2);

char MEMValid(void *x);
#define MEMValid(x) ( (int)(x)>=0 )

#endif _SYSLIB_MEM_H
