/*
	test program for PROGS SMS C library

	program to create the files for DLL's from one base file which is given
	as command line parameter (_dll is appended).
	The source file should contain (each on a line)
		name of extension
		name of the library
		version
		list of functions

	written by Joachim Van der Auwera for PROGS
	(c) 1994 by PROGS, Professional & Graphical Software
*/

#include "sms_h"
#include "io_h"
#include "str_h"

#define tell(x) IOPutString(chid,x,&sent)
#define put(ch,x) IOPutString(ch,x,&sent)

void report(char *oops)
{
	Error err;
	Channel chid;
	int sent; char chr;

	IOOpen("con",OPEN_OVER,&chid);
	tell(oops);
	tell("press a key");
	IOGet(chid,&chr);
	IOClose(chid);
}

void init(char *name)
{
	Error err;
	Channel in, lib, dll;
	int sent;
	char line[160]; char input[80];

	STRCopy(name,line);
	STRAppend("_dll",line);
	err=IOOpenPath(line,OPEN_READ,PATH_DATA,&in);
	if (err)
	{
		STRFormatPut(input,"can't open input file %s",line);
		report(input);
		return;
	}
	STRCopy(name,line);
	STRAppend("-dll_s",line);			/* part which should be linked in */
	err=IOOpenPath(line,OPEN_OVER,PATH_DATA,&lib);
	if (err) { report("can't open input file"); return; }
	STRCopy(name,line);
	STRAppend("-lib_s",line);			/* part used to define the library */
	err=IOOpenPath(line,OPEN_OVER,PATH_DATA,&dll);
	if (err) { report("can't open input file"); return; }

	put(lib, "; linking in this file will generate the links");
	put(lib, "; WARNING : this file assumes that all links are functions");
	STRFormatPut(line,"\t.text\n\t.even\n\t.ascii \"<<QD%s\"\n\tdc.l\t%s",
								"LL>>",name); put(lib,line);

	put(dll, "; use this file to generate the library");
	put(dll, "; WARNING : this file assumes that all links are functions");
	STRFormatPut(line,".globl\t_%s_start\n.globl\t_%s_end\n\t.text\n_%s_start:",
								name,name,name); put(dll,line);

	/* fill in extension name */
	IOGetString(in,input,80);
	while (strlen(input)<4) STRAppend (" ",input);
	if (strlen(input)!=4) {report("extension should be 4 chars long"); return; }
	STRFormatPut(line,"\t.ascii \"%s\"",input); put(lib,line);

	/* fill in thingname */
	IOGetString(in,input,80);
	STRFormatPut(line,"\tdc.w %d\n\t.ascii \"%s\"",strlen(input),input); put(lib,line);

	/* fill in version */
	IOGetString(in,input,80);
	if (*input)
	{
		STRFormatPut(line,"\t.ascii \" %s\"\n\n\t.bss\n%s:",
								input,name); put(lib,line);
	}

	/* now handle all the functions */
	while (IOGetString(in,input,80)!=ERR_EOF)
	{
		STRFormatPut(line,"%s:\n\tjmp\t_%s",input,input); put(dll,line);
		STRFormatPut(line,".globl _%s\n_%s:\n\tds.b\t6",input,input); put(lib,line);
	}

	STRFormatPut(line,"_%s_end:",name); put(dll,line);

	IOClose(in);
	IOClose(lib);
	IOClose(dll);
}
