/*
	test program for PROGS SMS C library

	Transform output file of inker into a resident extension.
	The file is searched on the data default only.
	Parameters : file to transform and message.

	written by Joachim Van der Auwera for PROGS
	(c) 1994 by PROGS, Professional & Graphical Software
*/

#include "sms_h"
#include "io_h"
#include "str_h"

#define tell(x) IOPutString(chid,x,&sent)
#define put(ch,x) IOPutString(ch,x,&sent)

#define HEADLEN 2

void report(char *oops)
{
	Error err;
	Channel chid;
	int sent; char chr;

	IOOpen("con",OPEN_OVER,&chid);
	tell(oops);
	tell("press a key");
	IOGet(chid,&chr);
	IOClose(chid);
}

Error init(char *param)
{
	Error err;
	Channel out;
	char line[80];
	Size orglen, len, space;
	char **list, *base;
	int sent;

	/* get the command line parameters */
	STRBreak(param,&list);
	if (*list==NULL) return ERR_ISYN;

	err=IOOpenPath(*list++,OPEN_OLD,PATH_DATA,&out);
	if (err)
	{
		STRFormatPutSafe(line,80,"can't open file %s",line);
		report(line);
		return err;
	}

	if (*list==NULL) return ERR_ISYN;

	err=IOLength(out, &orglen);
	if (err) { report("can't get length of input file"); return err; }
	err=IODataSpaceGet(out, &space);
	if (err) { report("can't get dataspace of input file"); return err; }
	len=orglen+HEADLEN+space+even(strlen(*list));
	err=MEMAllocate(len,&base);
	if (err) { report("can't allocate the memory"); return err; }

	/* fill in the header */
	base[0]=0x60; base[1]=HEADLEN-2+even(strlen(*list));
	MEMCopy(strlen(*list),*list,base+HEADLEN);

	err=IOFileLoad(out,orglen,base+HEADLEN+even(strlen(*list)));
	if (err) { report("can't load file"); return err; }

	/* now write the file back */
	err=IOPositionSet(out,0);		/* reset file position */
	if (err) { report("can't reset file"); return err; }
	err=IOPutBytes(out,len,base,&sent);
	if (err) { report("can't append file"); return err; }

	err=IOFileTypeSet(out,FILETYPE_NORMAL);
	if (err) { report("can't set filetype"); return err; }
	err=IODataSpaceSet(out,0);
	if (err) { report("can't clear dataspace"); return err; }
	
	return IOClose(out);
}
