/*
	test program for PROGS SMS C library

	Transform output file of linker into an external module.
	The file is searched on the data default only.
	Parameters : file to transform and module name.

	written by Joachim Van der Auwera for PROGS
	(c) 1994 by PROGS, Professional & Graphical Software
*/

#include "sms_h"
#include "io_h"
#include "str_h"

#define tell(x) IOPutString(chid,x,&sent)
#define put(ch,x) IOPutString(ch,x,&sent)

void report(char *oops)
{
	Error err;
	Channel chid;
	int sent; char chr;

	IOOpen("con",OPEN_OVER,&chid);
	tell(oops);
	tell("press a key");
	IOGet(chid,&chr);
	IOClose(chid);
}

Error init(char *param)
{
	Error err;
	Channel out;
	char line[80];
	Size orglen, len;
	char **list, *base;
	int sent;

	/* get the command line parameters */
	err=STRBreak(param,&list);
	if (err) return err;
	if (*list==NULL) return ERR_ISYN;

	err=IOOpenPath(*list++,OPEN_OLD,PATH_DATA,&out);
	if (err)
	{
		STRFormatPutSafe(line,80,"can't open file %s",line);
		report(line);
		return err;
	}

	err=IOLength(out, &orglen);
	if (err) { report("can't get length of input file"); return err; }
	len=orglen+even(strlen(*list)+1);
	err=MEMAllocate(len,&base);
	if (err) { report("can't allocate the memory"); return err; }

	/* fill in the header */
	STRCopy(*list,base);

	err=IOFileLoad(out,orglen,base+even(strlen(*list)+1));
	if (err) { report("can't load file"); return err; }

	/* now write the file back */
	err=IOPositionSet(out,0);		/* reset file position */
	if (err) { report("can't reset file"); return err; }
	err=IOPutBytes(out,len,base,&sent);
	if (err) { report("can't append file"); return err; }

	err=IOFileTypeSet(out,FILETYPE_EXTERNAL_MODULE);
	if (err) { report("can't set filetype"); return err; }
	
	return IOClose(out);
}
