/*
	test program for PROGS SMS C library
	test for str library
	written by Joachim Van der Auwera for PROGS
	(c) 1994 by PROGS, Professional & Graphical Software
*/

#include "sms_h"
#include "io_h"
#include "str_h"

#define tell(x) IOPutString(chid,x,&sent)

void init()
{
	Error err;
	Channel chid;
	int sent, t, res;
	char line[80],chr,str1[80],str2[80],*strptr,*pos;
	char **ptrlist;

	err=IOOpen("con",OPEN_OVER,&chid);

	/* strlen and STRLength */
	if (strlen("even kijken")!=11) tell("strlen fails");
	STRLength("even kijken",&t);
	if (t!=11) tell("STRLength fails");

	/* STRSameCD and STRSameCI */
	if (STRSameCD("hello","world")) tell("STRSameCD always TRUE");
	if (!STRSameCD("hello","hello")) tell("STRSameCD always FALSE");
	if (STRSameCD("hElLo","HeLlO")) tell("STRSameCD is CI");
	if (STRSameCD("aloh","aloha")) tell("STRSameCD compares 1 byte short (left)");
	if (STRSameCD("aloha","aloh")) tell("STRSameCD compares 1 byte short (right)");
	if (STRSameCI("hello","world")) tell("STRSameCI always TRUE");
	if (!STRSameCI("hello","hello")) tell("STRSameCI always FALSE");
	if (!STRSameCI("hElLo","HeLlO")) tell("STRSameCI is CD");
	if (STRSameCI("aloh","aloha")) tell("STRSameCI compares 1 byte short (left)");
	if (STRSameCI("aloha","aloh")) tell("STRSameCI compares 1 byte short (right)");

	/* STRCopy and STRCopySafe */
	STRCopy("this is the string I want to copy",line);
	if (!STRSameCD("this is the string I want to copy",line)) tell("MEMCopy fails");
	if (STRCopySafe("hello world",line,80)) tell("STRCopySafe always fails");
	if (STRCopySafe("hello world",line,12)) tell("STRCopySafe fails on exact fit");
	if (STRCopySafe("hello world",line,13)) tell("STRCopySafe byte too big fit");
	if (!STRCopySafe("hello world",line,11)) tell("STRCopySafe succeeds on byte too small fit");
	if (!STRCopySafe("hello world",line,5)) tell("STRCopySafe always succeeds");

	/* STRAppend and STRAppendSafe */
	STRCopy("start ",line);
	STRAppend("of string",line);
	if (!STRSameCD("start of string",line)) tell("STRAppend fails");
	STRCopy("start ",line);
	if (STRAppendSafe("of string",line,80)) tell("STRAppendSafe always fails");
	STRCopy("start ",line);
	if (STRAppendSafe("of string",line,16)) tell("STRAppendSafe fails on exact fit");
	STRCopy("start ",line);
	if (STRAppendSafe("of string",line,17)) tell("STRAppendSafe byte too big fit");
	STRCopy("start ",line);
	if (!STRAppendSafe("of string",line,15)) tell("STRAppendSafe succeeds on byte too small fit");
	STRCopy("start ",line);
	if (!STRAppendSafe("of string",line,5)) tell("STRAppendSafe always succeeds");

	/* STRUpper and STRLower */
	STRCopy("Ths should be cnvrted to pper cse",line);
	STRUpper(line); tell(line);
	STRCopy("tHIS SHOULD BE CNVRTED TO LOWER CSE",line);
	STRLower(line); tell(line);

	/* STRDuplicate */
	if (STRDuplicate("hello to this world",&strptr)) tell("can't STRDuplicate");
	if (!STRSameCD("hello to this world",strptr)) tell("STRDuplicate didn't generate duplicate");

	/* STRCompareCD and STRCompareCI */
	STRCompareCD("hello","world",&res);
	if (res!=-1) tell("STRCompareCD hello-world fails");
	STRCompareCD("hello","hello",&res);
	if (res) tell("STRCompareCD hello-hello fails");
	STRCompareCD("hElLo","HeLlO",&res);
	if (res==0) tell("STRCompareCD is CI");
	if (res==-1) tell("STRCompareCD reverses the answer");
	STRCompareCD("alo","aloha",&res);
	if (res!=-1) tell("STRCompareCD compares 1 byte short (left)");
	STRCompareCD("aloha","alo",&res);
	if (res!=1) tell("STRCompareCD compares 1 byte short (right)");
	STRCompareCI("hello","world",&res);
	if (res!=-1) tell("STRCompareCI hello-world fails");
	STRCompareCI("hello","hello",&res);
	if (res) tell("STRCompareCI hello-hello fails");
	STRCompareCI("hElLo","HeLlO",&res);
	if (res==1) tell("STRCompareCI is CD");
	if (res==-1) tell("STRCompareCI is CD and reverses the answer");
	STRCompareCI("alo","aloha",&res);
	if (res!=-1) tell("STRCompareCI compares 1 byte short (left)");
	STRCompareCI("aloha","alo",&res);
	if (res!=1) tell("STRCompareCI compares 1 byte short (right)");

	/* STRSearchCD and STRSearchCI */
	STRCopy("This is the string to test searching...",line);
	err=STRSearchCD("This",line,&pos);
	if (err) tell("oops, can't find string at start");
	if (pos!=line) tell ("pos filled in wrong at start");
	err=STRSearchCD("e str",line,&pos);
	if (err) tell("STRSeacrchCD fails");
	if (pos-10!=line) tell("STRSearchCD fills pos in wrongly");
	err=STRSearchCD("nothing",line,&pos);
	if (err!=ERR_ITNF) tell("STRSearchCD can find non-existing string");
	err=STRSearchCD("STRing",line,&pos);
	if (!err) tell("STRSeachCD is CI");
	err=STRSearchCD("strings",line,&pos);
	if (!err) tell("STRSearchCD compares a char too few of string to search");
	err=STRSearchCD("...",line,&pos);
	if (err) tell("STRSearchCD compares a byte too few in string to search in");
	err=STRSearchCI("This",line,&pos);
	if (err) tell("oops, can't find string at start");
	if (pos!=line) tell ("pos filled in wrong at start");
	err=STRSearchCI("e str",line,&pos);
	if (err) tell("STRSeacrchCI fails");
	if (pos-10!=line) tell("STRSearchCI fills pos in wrongly");
	err=STRSearchCI("nothing",line,&pos);
	if (err!=ERR_ITNF) tell("STRSearchCI can find non-existing string");
	err=STRSearchCI("STRing",line,&pos);
	if (err) tell("STRSeachCI is CD");
	err=STRSearchCI("strings",line,&pos);
	if (!err) tell("STRSearchCI compares a char too few of string to search");
	err=STRSearchCI("...",line,&pos);
	if (err) tell("STRSearchCI compares a byte too few in string to search in");

	/* STRBreak */
	STRCopy("let's \\see \"if \"\"this\"\"\" /works -properly",line);
	tell("STRBreak"); tell(line); tell ("into");
	STRBreak(line,&ptrlist);
	for(t=0;ptrlist[t];t++)
	{
		tell(ptrlist[t]);
	}

	tell ("press a key");
	IOGet(chid,&chr);
	IOClose(chid);
}
