/*
	test program for PROGS SMS C library
	test for STRFormatGet and STRFormatPut library
	written by Joachim Van der Auwera for PROGS
	(c) 1994 by PROGS, Professional & Graphical Software
*/

#include "sms_h"
#include "io_h"
#include "str_h"
#include "math_h"

#define tell(x) IOPutString(chid,x,&sent)

#define gettell(x) STRFormatGet("123.45e6",NULL,x,&doubledata); \
				   STRFormatPut(line,x,doubledata); \
				   tell(line);


void init()
{
	Error err;
	Channel chid;
	int sent, t, res, intdata, count;
	double doubledata;
	char line[80],chr,str1[80],str2[80],*strptr,*pos;
	char **ptrlist, chardata, stringdata[40];

	err=IOOpen("con",OPEN_OVER,&chid);

	/* STRFormatPut */
	STRFormatPut(line, "integer %d %#d %+d", 1234, 1234, 1234); tell(line);
	STRFormatPut(line, "octal %o %#o", 1234, 1234); tell(line);
	STRFormatPut(line, "hex %x %#x", 1234, 1234); tell(line);
	STRFormatPut(line, "unsigned %u neg %u", 1234, -1234); tell(line);
	STRFormatPut(line, "char %c string %s", 'A', "hello world"); tell(line);
	STRFormatPut(line, "percent %% pointer %p", "hello world"); tell(line);
	STRFormatPut(line, "f %f g %g e %e", M_PI, M_PI, M_PI); tell(line);
	STRFormatPut(line, "F %F G %G E %E", M_PI, M_PI, M_PI); tell(line);
	STRFormatPut(line, "f %8.3f %+8.3f %-8.3f %08.3f", M_PI, M_PI, M_PI, M_PI); tell(line);
	STRFormatPut(line, "g %8.3g %+8.3g %-8.3g %08.3g", M_PI, M_PI, M_PI, M_PI); tell(line);
	STRFormatPut(line, "e %8.3e %+8.3e %-8.3e %08.3e", M_PI, M_PI, M_PI, M_PI); tell(line);
	STRFormatPut(line, "f %8.3f %+8.3f %-8.3f %08.3f", 123.45, 123.45, 123.45, 123.45); tell(line);
	STRFormatPut(line, "g %8.3g %+8.3g %-8.3g %08.3g", 123.45, 123.45, 123.45, 123.45); tell(line);
	STRFormatPut(line, "e %8.3e %+8.3e %-8.3e %08.3e", 123.45, 123.45, 123.45, 123.45); tell(line);

	/* STRFormatPutSafe */
	err=STRFormatPutSafe(line,30,"test %s","hello world."); tell(line);
	if (err) tell("STRFormatPutSafe fails on too large fit");
	err=STRFormatPutSafe(line,18,"test %s","hello world.");
	if (err) tell("STRFormatPutSafe fails on exact fit");
	err=STRFormatPutSafe(line,19,"test %s","hello world.");
	if (err) tell("STRFormatPutSafe fails on one too large fit");
	err=STRFormatPutSafe(line,4,"test %s","hello world.");
	if (!err) tell("STRFormatPutSafe works on too small fit");
	err=STRFormatPutSafe(line,17,"test %s","hello world.");
	if (!err) tell("STRFormatPutSafe works on 1 too small fit");

	/* STRFormatGet */
	STRFormatGet("1234",&count,"%d",&intdata);
	if (intdata!=1234) tell("STRFormatGet %d doesn't work");
	if (count!=1) tell("STRFormatGet fills count wrong");
	STRFormatGet("1342",NULL,"%i",&intdata);
	if (intdata!=1342) tell("STRFormatGet %i doesn't work");
	STRFormatGet("010",NULL,"%i",&intdata);
	if (intdata!=010) tell("STRFormatGet %i-octal doesn't work");
	STRFormatGet("0x10",NULL,"%i",&intdata);
	if (intdata!=0x10) tell("STRFormatGet %i-hex doesn't work");
	STRFormatGet("10",NULL,"%o",&intdata);
	if (intdata!=010) tell("STRFormatGet %o doesn't work");
	STRFormatGet("10",NULL,"%x",&intdata);
	if (intdata!=0x10) tell("STRFormatGet %x doesn't work");
	STRFormatGet("hello world",NULL,"%c%s",&chardata,stringdata);
	if (chardata!='h') tell("STRFormatGet %c doesn't work");
	if (!STRSameCD(stringdata,"ello")) tell("STRFormatGet %s doesn't work");
	STRFormatPut(line,"%c%s",chardata,stringdata); tell(line);
	STRFormatGet("test 4321",NULL,"test %d",&intdata);
	if (intdata!=4321) tell("STRFormatGet matching fails");
	gettell("%f");
	gettell("%g");
	gettell("%e");

	tell ("press a key");
	IOGet(chid,&chr);
	IOClose(chid);
}
